    processor  6502
    include    vcs.h

;----------------------------------------------------------
; 4K GAME
;----------------------------------------------------------

MAXOBJECTS  = $0A
LINES       = $40

    SEG.U vars
    ORG $80

; Both Banks:
gameState       ds 1                ; X/X/X/X/X/X/X/INTRO
bcdScore        ds 3                ; Last/Current score
bcdWave         ds 1                ; desired/current wave
frameCounter    ds 1                ; Incremented every frame

; This Bank:
xpos            ds #MAXOBJECTS      ; horizontal position of sprites
ypos            ds #MAXOBJECTS      ; vertical position of sprites
xmov            ds #MAXOBJECTS      ; horizontal motion for sprite
ymov            ds #MAXOBJECTS      ; vertical motion for sprite
sprtsizetype    ds #MAXOBJECTS      ; list of sprite colors
radarX          ds #MAXOBJECTS-1    ; horizontal position of radar
radarY          ds #MAXOBJECTS-1    ; vertical position of radar
vline           ds 1                ; current vertical line being drawn
p0count         ds 1                ; number of lines left to draw for player 0
tieshapeptr     ds 2                ;
tieshapeptr2    ds 2                ;
expCounter      ds 1                ; Decremented every frame
shieldState     ds 1                ; 0-5
starxpos        ds 1                ;
starPointer     ds 2                ; Pointer to star pattern
tempVar1        ds 1                ; temp variable
tempVar2        ds 1                ; temp variable
tempVar3        ds 1                ; temp variable

    SEG     Bank0
    ORG     $1000
    RORG    $D000

MainScreen
;-----------------------------------------------------------
;SPACE DISPLAY KERNEL TOP
;-----------------------------------------------------------

    STA WSYNC
    STA HMOVE
    JSR AnyRTS
    JSR AnyRTS
    LDX tempVar1
    LDA #$00
    STA COLUBK
    LDA #(LINES*2)+1          ; number of lines to draw on screen
    STA vline
    STA tempVar1    ; Init p0count
    LDA tempVar2
    STA HMP0
    LDA tempVar3
    STA HMP1
    STX tempVar2
    LDA #%11110000
    STA PF0
    LDA #%10000000
    STA PF1
    STY PF2

;-----------------------------------------------------------
;SPACE DISPLAY KERNEL
;-----------------------------------------------------------

NextLine
    STA WSYNC           ; Wait for horizontal sync
    STA HMOVE           ; Horizontal Movement

; Draw starfield
    LDY vline
    LDA (starPointer),Y ;
    STA ENABL

; To reposition or not to reposition?
    LDA tempVar1        ; Are we to reposition a sprite?
    BMI Notstart        ; N: Skip first kernel part

; Draw crosshair
    LDA #$00
    CPY #$41
    BNE NoCrossHair
    LDA #$40
NoCrossHair
    STA PF2
    
; Two lines *wasted* with repositioning...
    DEY        
    DEY
    STY vline

; Set new ships size
    LDY sprtsizetype,X
    LDA shapeheight,Y
    STA p0count

; Prepare Positioning ship
    LDY xpos,X
    LDA movetab1,Y
    STA HMP0
    LDA movetab2,Y
    STA HMP1
    AND #$0F
    TAY
; Clear sprites
    LDA #$00
    STA WSYNC
    STA HMOVE           ; Horizontal Movement
    STA.w ENABL ; Waste 1 cycle
; Go! Position it!  
; This _must_ start on cycle 0, to match with our tables!!!
PosLoop
    DEY
    BPL PosLoop
    STA RESP0
    STA RESP1
    STY tempVar1
    JMP NextLine

SkipDraw
    LDA shapeclrtab,Y   ; Get sprite color
    STA COLUP0
    STA COLUP1
    LDA #$00
    STA WSYNC
    STA HMOVE           ; Horizontal Movement
    STA GRP0
    STA GRP1
    JMP NotDraw

Notstart
    LDA #$00
    STA HMP0
    STA HMP1

; Draw crosshair
    CPY #$41
    BNE NoCrossHair2
    LDA #$40
NoCrossHair2
    STA PF2

    LDY sprtsizetype,X
    DEC p0count
    BMI SkipDraw
; Set shapes
    LDA shapeptrtab,Y
    STA tieshapeptr     ; Set space object pointer
    LDA shapeptrtab2,Y
    STA tieshapeptr2     ; Set space object pointer
    LDY p0count
    LDA (tieshapeptr),y
    STA GRP0
    STA WSYNC
    STA HMOVE           ; Horizontal Movement
    LDA (tieshapeptr2),y
    STA GRP1
NotDraw
    LDY vline
    DEY
    LDA (starPointer),Y ;
    STA ENABL           ; En/disable Starfield during kernel


; Ok, the miraculous skipping code 
; I'm trying to comment that as good as possible... :-)

    CPY #$03            ; Y: Are we already to close to the bottom?
    BCC Continue        ; Y: -> Bail out

    LDA xpos,X          ; Next sprite invisible?
    BMI ContinueInv     ; -> Skip to next sprite
    LDA p0count         ; If we are already drawing a sprite
    BPL Continue        ; -> bail out completely
    TYA
    LSR                 ; Sprites have 1/2 the Starfield resolution
    CMP ypos,X          ; Check if sprite starts now
    BNE CheckDoable     ; N: Check if we already passed it's line
    LDA #$00            ;
    STA tempVar1        ; N: Set reposition flag
    BEQ Continue        ; Bail out
CheckDoable
    BPL Continue        ; We've to wait some more for this sprite...
Continue2
    DEX                 ; Oh, we're too late for this one, try next
    BPL Continue        ;
    LDX #$00            ; We should never have an index < 0
Continue
    DEY                 ; Decrease scannline by one
    STY vline           ; Store it away...
    BMI Radar
    JMP NextLine

; We eventually manage to skip away two invisible sprites in a row
ContinueInv
    DEX                 ; Oh, we're too late for this one, try next
    BPL ContinueInv2    ;
    LDX #$00            ; We should never have an index < 0
ContinueInv2
    LDA xpos,X          ; Next sprite invisible?
    BMI Continue2       ; -> Skip to next sprite
    BPL Continue

;-----------------------------------------------------------
; RADAR DISPLAY KERNEL
;-----------------------------------------------------------

Radar
    STA WSYNC
    STA HMOVE
    STY PF1
    STY PF2
    INY
    STY GRP0
    STY GRP1
    STY ENABL
    LDA #$1F
    STA vline
    LDA #$05
    STA NUSIZ1
    LDA #$1A
    STA COLUP1
    LDA #$B0
    STA HMP1
    STA RESP1
    LDA #$44
    STA COLUP0

    STA WSYNC           ; Wait for horizontal sync
    STA HMOVE           ; Horizontal Movement
    JSR AnyRTS
    LDA #$8A
    STA COLUBK
    JSR AnyRTS
    STA HMCLR

    STA WSYNC           ; Wait for horizontal sync
    STA HMOVE           ; Horizontal Movement
    LDX #MAXOBJECTS-2
GetStartingIndex
    LDA radarY,X

    BPL GetIndexDone
    DEX
    CPX #$04
    BNE GetStartingIndex

GetIndexDone
    LDA vline

NextLine2
    STA WSYNC           ; Wait for horizontal sync
    STA HMOVE           ; Horizontal Movement
    STY ENAM0

; To reposition or not to reposition?
    CMP radarY,X
    BNE Notstart2        ; N: Skip first kernel part
    
    TAY
    LDA radarsprite,Y
    STA GRP1
    NOP

    LDY radarX,X
    LDA movetab4,Y
    STA HMM0
    AND #$0F
    TAY
; Go! Position it!  
PosLoop3
    DEY
    BPL PosLoop3
    STA RESM0
    DEC vline
    LDA #$0F
    STA PF2
    LDA vline
    BPL NextLine2
    BMI RadarDone

Notstart2
    BCS RadarOk
    DEX
    BPL RadarOk
    LDX #$00
RadarOk
    TAY
    LDA radarsprite,Y
    STA GRP1
    LDY #$00
    STY HMM0
    JSR AnyRTS
    NOP
    DEC vline           ; Store it away...
    LDA #$0F
    STA PF2
    LDA vline
    BPL NextLine2

RadarDone
    STA WSYNC
    STA HMOVE           ; Horizontal Movement
    LDA #$00
    STA NUSIZ1
    STA ENAM0
    LDA #$FF
    STA PF2
    LDY #$04
    JSR CockpitFrame

    LDA #$02            ;
    STA VBLANK          ;   start vertical blanking

    LDA #$23
    STA TIM64T          ; 30 lines overscan

;-----------------------------------------------------------
; BANKSWITCH 
;-----------------------------------------------------------

    LDA SWCHB
    LSR
    TAY
    BCS .Continue3
    LDA #$01
    STA gameState
    JMP SwitchToBank1

.Continue3
    TYA
    LSR
    BCS .Continue2
    JMP SwitchToBank1

.Continue2
    JSR Joy

;-----------------------------------------------------------
; MOVE ALL SPACE OBJECTS
;-----------------------------------------------------------

    LDA frameCounter
    LSR 
    BCS NoMove
    LDX #MAXOBJECTS-1
MoveNextSprite
    JMP SizeOk ; Debugging
    LDA frameCounter
    AND #$0F
    BNE SizeOk
    LDA sprtsizetype,x
    CMP #$30
    BEQ SizeOk
    LDA sprtsizetype,x
    AND #%11111000
    STA tempVar1
    INC sprtsizetype,x
    LDA sprtsizetype,x
    AND #$07
    ORA tempVar1
    STA sprtsizetype,x
SizeOk
    LDA xpos,x
    CLC
    ADC xmov,x
    STA xpos,x
    LDA ypos,x
    CLC
    ADC ymov,x
    AND #$7F
    STA ypos,x
    DEX
    BPL MoveNextSprite

NoMove:
    JMP OverscanLoop

;-----------------------------------------------------------
; CARTRIDGE MAIN ENTRY & INIT
;-----------------------------------------------------------

MainGame
    LDX #$7F
    LDA #$00
.clearLoop:
    STA VSYNC,X
    dex
    bne     .clearLoop

    LDA #$00
    STA gameState

    SED
    LDA bcdWave
    CLC
    ADC #$01
    STA bcdWave
    CLD
    
    LDA #$08        ; reflect spaceships
    STA REFP1

    LDA #$0C
    STA COLUPF

    LDA #$05
    STA CTRLPF
    STA shieldState

    LDA #>starenamtab
    STA starPointer+1

    LDA #>tieshape08
    STA tieshapeptr+1
    LDA #>tieshape08
    STA tieshapeptr2+1

; Init sprites
    ldx     #MAXOBJECTS-1
InitSprites
    lda     isprypos,x
    sta     ypos,x
    lda     isprxpos,x
    sta     xpos,x
    lda     iymov,x
    sta     ymov,x
    lda     ixmov,x
    sta     xmov,x
    lda     isprtsizetype,x
    sta     sprtsizetype,x
    dex
    bpl     InitSprites

OverscanLoop
    LDA     INTIM
    BNE     OverscanLoop

;-----------------------------------------------------------
; SCREEN MAIN LOOP START WITH VSYNC
;-----------------------------------------------------------

    LDY #$02
    LDA #$00
    STY WSYNC           ; Finish current line
    STY VSYNC           ; start vertical sync
    STY WSYNC           ; Finish current line
    STY WSYNC           ; Finish current line
    STY WSYNC           ; Finish current line
    STA VSYNC           ; Stop vertical sync

;-----------------------------------------------------------
; ADJUST STARFIELD
;-----------------------------------------------------------

    LDA #$7F    ;2
    AND starPointer     ;3
    STA starPointer     ;3
    LDA starxpos     ;3
    CMP #$A0    ;2
    BCC LF3B1   ;2
    CMP #$C0    ;2
    BCS LF3AF   ;2
    SBC #$9F    ;2
    BCS LF3B1   ;2

LF3AF: 
    ADC #$9F    ;2

LF3B1: 
    STA starxpos     ;3

;-----------------------------------------------------------
; HANDLE COUNTERS/TIMERS
;-----------------------------------------------------------

    LDA #$2C
    STA TIM64T  ; set timer for next wait

    INC frameCounter
    BNE ShieldOk
    DEC shieldState
    BPL ShieldOk
    LDA #$05
    STA shieldState
ShieldOk
    DEC expCounter

    JSR FlickerSort

;-----------------------------------------------------------
; POSITION STARFIELD
;-----------------------------------------------------------

    LDY starxpos
    LDA movetab3,Y
    STA HMBL
    AND #$0F
    TAY
    STA WSYNC
PosStarfield
    DEY
    BPL PosStarfield
    STA RESBL
    STA WSYNC
    STA HMOVE
    STA WSYNC
    LDA #$00    ; Clear Ball movement register
    STA HMBL    ; Before repositioning Half-Sprite

    LDA #$0     ;
    STA p0count ; Assume no half-sprite

; Loop over all space objects, try finding the top 
; half-sprite, if it exists

    LDX #MAXOBJECTS-1   ; all objects

SearchHalfSprite
    LDA xpos,x
    BMI TryNext         ; Sprite invisible, bail out
    LDY sprtsizetype,X  ; Preload Y with type
    LDA ypos,X          ; Sprite start already below screen start
    CMP #$42            ; Sprite starts on top of screen 
    BCC Loop2           ; Bail out!
    SBC shapeheight,Y   ; ypos-height
    STA tempVar1        ; Store temporary
    CMP #$42            ;
    BCC DoHalfSprite    ; Sprite not done before screen starts? Gotcha!

TryNext
    DEX                 ;
    BPL SearchHalfSprite; continue search

    LDX #MAXOBJECTS-1   ; we didn't find any half-sprite, set X

Loop2
    LDY     INTIM
    BNE     Loop2   ;   waste time

    LDA #$80            ;
    STA HMBL            ; Set constant starfield movement 
    STX tempVar1

    STA WSYNC
    STA HMOVE
    STY VBLANK          ; End vertical blanking
    DEY
    STY PF0
    STY PF1
    STY PF2
    LDX shieldState
    LDA #$14
    SEC
    SBC shieldloop,X
    TAY
    JSR CockpitFrame
    
    LDY #$00
ShieldLine
    STA WSYNC
    STA HMOVE
    LDA shieldcolor,X
    STA COLUBK
    LDA #$FF
    CPX #$05
    BNE PF1ValOk
    LDA #$00
PF1ValOk
    ORA shieldpattern1,Y
    STA PF1
    LDA shieldpattern2,Y
    ORA shieldor,X
    STA PF2
    INY
    TYA
    CMP shieldloop,X
    BNE ShieldLine

    STA WSYNC
    STA HMOVE
    LDA #$FF
    STA PF1
    STA PF2
    JMP MainScreen

DoHalfSprite
    LDA #$42            ; Calculate drawable size of the half-sprite
    SBC tempVar1
FullSprite
    STA p0count         ; Store the remaing sprite lines
    LDA shapeclrtab,Y   
    STA COLUP0
    STA COLUP1          ; Set half-sprite color
    LDY xpos,X          ; Position half-sprite:
    LDA movetab1,Y
    STA tempVar2
    LDA movetab2,Y
    STA tempVar3
    AND #$0F
    TAY
    STA WSYNC
    STA HMOVE
    NOP
    NOP

    ; Go! Position half-sprite!  
PosLoop2
    DEY
    BPL PosLoop2
    STA RESP0
    STA RESP1
    JMP Loop2           ; Kick into display kernel

;-----------------------------------------------------------
;FLICKER SORT SUB
;-----------------------------------------------------------

FlickerSort:
    LDX #MAXOBJECTS-2
SortLoop:
    LDY sprtsizetype+1,X    ; A-> 1st sprite type and size
    LDA ypos+1,X            ; A-> 1st sprite ypos
    SEC                     ; 
    SBC shapeheight,Y       ; A-> 1st sprite ypos - size
    CMP ypos,X              ; 2nd Sprite drawable?
    BPL DontSwap            ; Y: All right!
SwapThis
    LDA xpos,X              ; N: Swap horizontal position
    LDY xpos+1,X
    STA xpos+1,X
    STY xpos,X
    LDA ypos,X                ; Swap vertical position
    LDY ypos+1,X
    STA ypos+1,X
    STY ypos,X
    LDA xmov,X             ; Swap horizontal movement
    LDY xmov+1,X
    STA xmov+1,X
    STY xmov,X
    LDA ymov,X             ; Swap vertical movement
    LDY ymov+1,X
    STA ymov+1,X
    STY ymov,X
    LDA sprtsizetype,X             ; Swap size & type
    LDY sprtsizetype+1,X
    STA sprtsizetype+1,X
    STY sprtsizetype,X
DontSwap:
    DEX
    BPL SortLoop

; Search border Y value
    LDX #MAXOBJECTS-1
RadarFind
    LDA ypos,X
    CLC 
    ADC #$20
    BPL YValueFound
    DEX
    BNE RadarFind
YValueFound

; Prepare Radar
    LDY #MAXOBJECTS-2
RadarSort
    LDA sprtsizetype,X
    CMP #$0B
    BCC NextRadarElem
    CMP #$18
    BCC SkipShip
    LDA xpos,X
    CLC 
    ADC #$40
    LSR
    LSR
    LSR
    STA radarX,Y
    LDA ypos,X
    CLC 
    ADC #$20
    AND #%01111111
    LSR
    LSR
.Continue
    STA radarY,Y
NextRadarElem
    LDA radarshifttab,X
    TAX
    DEY
    BPL RadarSort
AnyRTS
    RTS

SkipShip
    LDA #$FF
    BMI .Continue

radarshifttab
    .byte #$09,#$00,#$01,#$02,#$03,#$04,#$05,#$06,#$07,#$08
;-----------------------------------------------------------
;JOYSTICK READING SUB
;-----------------------------------------------------------

Joy     
        LDA #$00
        STA tempVar1
        STA tempVar2
        STA tempVar3

        LDA SWCHA
        ASL
        TAY
        BCS RightDone
        DEC starxpos
        LDA #$FF
        STA tempVar1
RightDone
        TYA
        ASL
        TAY
        BCS LeftDone
        INC starxpos
        LDA #$01
        STA tempVar1
LeftDone
        TYA
        ASL
        TAY
        BCS DownDone
        LDA starxpos
        SEC
        SBC #$10
        STA starxpos
        INC starPointer
        INC starPointer
        LDA #$FF
        STA tempVar2
DownDone
        TYA
        ASL
        BCS UpDone
        LDA starxpos
        CLC
        ADC #$10
        STA starxpos
        DEC starPointer
        DEC starPointer
        LDA #$01
        STA tempVar2
UpDone

; Object Handler : 1. Laser

    LDX #MAXOBJECTS-1

MoveNextSprite2
    LDA sprtsizetype,X
    CMP #$0C
    BPL NoLaserSprite
    LDA xpos,X          ; A: 0E-6E
    BMI CheckFire       

    SEC
    SBC #$0E            ; A: 00-60
    LSR
    LSR                 
    LSR                 
    LSR                 ; A: 00-06
    TAY
    
    LDA sprtsizetype,X
    AND #$03
    ORA shottab,Y
    STA sprtsizetype,X
  
    LDA xpos,X
    CMP #$38
    BEQ ExpiredNow
    CMP #$44
    BEQ ExpiredNow
    CMP #$3E
    BEQ ExpiredNow2

JumpDone
    JMP Spritedone

ExpiredNow
    LDA #$FF
    STA tempVar3
    BMI JumpDone

ExpiredNow2
    LDA #$FF
    STA tempVar3

SecureInvisibility
    LDA #$D0
    STA xpos,X
    STA ypos,X
    BNE JumpDone

CheckFire
    LDA INPT4
    BMI SecureInvisibility
    INC sprtsizetype,X
    LDA sprtsizetype,X
    AND #$03
    STA sprtsizetype,X
    TAY
    LDA shotxpos,Y
    STA xpos,X
    LDA shotypos,Y
    STA ypos,X
    LDA shotymov,Y
    STA ymov,X
    LDA shotxmov,Y
    STA xmov,X
    BNE JumpDone

NoLaserSprite
    LDA xpos,x
    CLC
    ADC tempVar1
    STA xpos,x
    LDA ypos,x
    CLC
    ADC tempVar2
    AND #$7F
    STA ypos,x

    LDA sprtsizetype,X
    AND #%00111000
    CMP #%00010000
    BNE NoExplosionSprite
    LDA expCounter
    BEQ ExplosionDone
    AND #%00000111
    BNE NoExplosionSprite
    LDA sprtsizetype,X
    EOR #%00000001
    STA sprtsizetype,X

NoExplosionSprite
    LDY sprtsizetype,X
    LDA #$46
    SBC xpos,X
    BMI NoHit
    SBC #$10
    BPL NoHit
    LDA #$22
    ADC shapeheight,Y
    SBC ypos,X
    BMI NoHit
    SBC shapeheight,Y
    BPL NoHit

    LDA tempVar3        ; Shot expired this frame?
    BEQ NoHit
    LDA sprtsizetype,X
    AND #%00000110
    ORA #%00010000
    STA sprtsizetype,X
    LDA #$00
    STA xmov,X
    STA ymov,X
    LDA #$40
    STA expCounter
NoHit
    LDA sprtsizetype,X
    CMP #$0F
    BNE Spritedone
    LDA xpos,X
    ORA #%10000000
    STA xpos,X
Spritedone
    DEX
    BMI QuitMovement
    JMP MoveNextSprite2
QuitMovement
    RTS

ExplosionDone
    LDA #$0F
    STA sprtsizetype,X
    STA ypos,X
    BPL Spritedone

CockpitFrame
    STA WSYNC
    STA HMOVE
    DEY
    BNE CockpitFrame
    RTS

;-----------------------------------------------------------
;DATA BLOCK
;-----------------------------------------------------------

    ORG     $1B00
    RORG    $DB00

; All space objects are organized here in this page

shapeheight
    .byte $05,$05,$05,$05,$04,$04,$04,$04
    .byte $03,$03,$03,$03,$07,$07,$07,$00
    .byte $03,$03,$06,$06,$0A,$0A,$0D,$0D
    .byte $01,$02,$03,$04,$06,$08,$0C,$0C
    .byte $01,$02,$03,$04,$06,$08,$0C,$0C
    .byte $01,$03,$05,$07,$09,$0B,$0D,$0F
    .byte $01,$03,$05,$07,$09,$0B,$0D,$0F
    .byte $01,$03,$05,$07,$09,$0B,$0D,$0F

shapeclrtab
    .byte   $40,$40,$40,$40,$42,$42,$42,$42
    .byte   $44,$44,$44,$44,$34,$34,$34,$34
    .byte   $40,$2E,$40,$2E,$40,$2E,$40,$2E
    .byte   $80,$80,$82,$82,$84,$84,$86,$86
    .byte   $80,$80,$82,$82,$84,$84,$86,$86
    .byte   $80,$80,$82,$82,$84,$84,$86,$86
    .byte   $80,$80,$82,$82,$84,$84,$86,$86
    .byte   $80,$80,$82,$82,$84,$84,$86,$86

shapeptrtab
    .byte  #<void,#<laser03,#<laser03,#<void,#<void,#<laser02,#<laser02,#<void
    .byte  #<void,#<laser01,#<laser01,#<void,#<mothership01,#<mothership01,#<mothership01,#<void
    .byte  #<explosion02,#<explosion02,#<explosion04,#<explosion04,#<explosion06,#<explosion06,#<explosion08,#<explosion08
    .byte  #<meteor01,#<meteor02,#<meteor03,#<meteor04,#<meteor05,#<meteor06,#<meteor07,#<meteor08
    .byte  #<shot01,#<shot02,#<shot03,#<shot04,#<shot05,#<shot06,#<shot07,#<shot08
    .byte  #<tieshape01,#<tieshape02,#<tieshape03,#<tieshape04,#<tieshape05,#<tieshape06,#<tieshape07,#<tieshape08
    .byte  #<tieshape01,#<tieshape02,#<tieshape03,#<tieshape04,#<tieshape05,#<tieshape06,#<tieshape07,#<tieshape08
    .byte  #<tieshape01,#<tieshape02,#<tieshape03,#<tieshape04,#<tieshape05,#<tieshape06,#<tieshape07,#<tieshape08

shapeptrtab2
    .byte  #<laser03,#<void,#<void,#<laser03,#<laser02,#<void,#<void,#<laser02
    .byte  #<laser01,#<void,#<void,#<laser01,#<mothership02,#<mothership02,#<mothership02,#<void
    .byte  #<explosion202,#<explosion202,#<explosion204,#<explosion204,#<explosion206,#<explosion206,#<explosion208,#<explosion208
    .byte  #<meteor01,#<meteor02,#<meteor03,#<meteor04,#<meteor05,#<meteor06,#<meteor07,#<meteor08
    .byte  #<shot01,#<shot02,#<shot03,#<shot04,#<shot05,#<shot06,#<shot07,#<shot08
    .byte  #<tieshape01,#<tieshape02,#<tieshape03,#<tieshape04,#<tieshape05,#<tieshape06,#<tieshape07,#<tieshape08
    .byte  #<tieshape01,#<tieshape02,#<tieshape03,#<tieshape04,#<tieshape05,#<tieshape06,#<tieshape07,#<tieshape08
    .byte  #<tieshape01,#<tieshape02,#<tieshape03,#<tieshape04,#<tieshape05,#<tieshape06,#<tieshape07,#<tieshape08

; All space object shapes are stored here in this page

tieshape08
       .byte %10000000
       .byte %11000000
       .byte %11000000
       .byte %11000000
       .byte %11000000
       .byte %11000011
       .byte %11111111
       .byte %11111110
       .byte %11111111
       .byte %11000011
       .byte %11000000
       .byte %11000000
       .byte %11000000
       .byte %11000000
       .byte %10000000

tieshape07
       .byte %01000000
       .byte %01000000
       .byte %01000000
       .byte %01000000
       .byte %01100011
       .byte %01111111
       .byte %01111110
       .byte %01111111
       .byte %01100011
       .byte %01000000
       .byte %01000000
       .byte %01000000
       .byte %01000000

tieshape06
       .byte %00100000
       .byte %00100000
       .byte %00100000
       .byte %00100000
       .byte %00110011
       .byte %00111111
       .byte %00110011
       .byte %00100000
       .byte %00100000
       .byte %00100000
       .byte %00100000

tieshape05
       .byte %00010000
       .byte %00010000
       .byte %00010000
       .byte %00010001
       .byte %00011111
       .byte %00010001
       .byte %00010000
       .byte %00010000
       .byte %00010000

tieshape04
       .byte %00001000
       .byte %00001000
       .byte %00001001
       .byte %00001111
       .byte %00001001
       .byte %00001000
       .byte %00001000

tieshape03
       .byte %00000100
       .byte %00000101
       .byte %00000111
       .byte %00000101
       .byte %00000100

tieshape02
       .byte %00000010
       .byte %00000011
       .byte %00000010

tieshape01
       .byte %00000001

meteor08:
       .byte $03 ; |      XX| $FEC1
       .byte $0F ; |    XXXX| $FEC2
       .byte $0F ; |    XXXX| $FEC4
       .byte $1F ; |   XXXXX| $FEC5
       .byte $1F ; |   XXXXX| $FEC7
       .byte $3F ; |  XXXXXX| $FEC8
       .byte $3F ; |  XXXXXX| $FEC9
       .byte $1F ; |   XXXXX| $FECB
       .byte $1F ; |   XXXXX| $FECC
       .byte $0F ; |    XXXX| $FECD
       .byte $0F ; |    XXXX| $FECF
       .byte $03 ; |      XX| $FED0

meteor07:
       .byte $01 ; |       X| $FED7
       .byte $07 ; |     XXX| $FED8
       .byte $07 ; |     XXX| $FED9
       .byte $0F ; |    XXXX| $FEDA
       .byte $0F ; |    XXXX| $FEDB
       .byte $1F ; |   XXXXX| $FEDC
       .byte $1F ; |   XXXXX| $FEDD
       .byte $0F ; |    XXXX| $FEDE
       .byte $0F ; |    XXXX| $FEDF
       .byte $07 ; |     XXX| $FEE0
       .byte $07 ; |     XXX| $FEE1
       .byte $01 ; |       X| $FEE2

meteor06:
       .byte $01 ; |       X| $FE7A
       .byte $07 ; |     XXX| $FE7C
       .byte $07 ; |     XXX| $FE7C
       .byte $0F ; |    XXXX| $FE7D
       .byte $0F ; |    XXXX| $FE7E
       .byte $07 ; |     XXX| $FE7F
       .byte $07 ; |     XXX| $FE7C
       .byte $01 ; |       X| $FE81

meteor05:
       .byte $01 ; |       X| $FE6E
       .byte $03 ; |      XX| $FE6F
       .byte $07 ; |     XXX| $FE70
       .byte $07 ; |     XXX| $FE71
       .byte $03 ; |      XX| $FE72
       .byte $01 ; |       X| $FE73

meteor04:
       .byte $01 ; |       X| $FE64
       .byte $03 ; |      XX| $FE65
       .byte $03 ; |      XX| $FE66
       .byte $01 ; |       X| $FE67

meteor03:
       .byte $01 ; |       X| $FE5B
       .byte $03 ; |      XX| $FE5C
       .byte $01 ; |       X| $FE5D

meteor02:
       .byte $01 ; |       X| $FE4A
       .byte $01 ; |       X| $FE4B

meteor01:
       .byte $01 ; |       X| $FE41

shot08:
       .byte $03 ; |      XX| $FEC1
       .byte $0F ; |    XXXX| $FEC2
       .byte $0F ; |    XXXX| $FEC4
       .byte $1F ; |   XXXXX| $FEC5
       .byte $1F ; |   XXXXX| $FEC7
       .byte $3F ; |  XXXXXX| $FEC8
       .byte $3F ; |  XXXXXX| $FEC9
       .byte $1F ; |   XXXXX| $FECB
       .byte $1F ; |   XXXXX| $FECC
       .byte $0F ; |    XXXX| $FECD
       .byte $0F ; |    XXXX| $FECF
       .byte $03 ; |      XX| $FED0

shot07:
       .byte $01 ; |       X| $FED7
       .byte $07 ; |     XXX| $FED8
       .byte $07 ; |     XXX| $FED9
       .byte $0F ; |    XXXX| $FEDA
       .byte $0F ; |    XXXX| $FEDB
       .byte $1F ; |   XXXXX| $FEDC
       .byte $1F ; |   XXXXX| $FEDD
       .byte $0F ; |    XXXX| $FEDE
       .byte $0F ; |    XXXX| $FEDF
       .byte $07 ; |     XXX| $FEE0
       .byte $07 ; |     XXX| $FEE1
       .byte $01 ; |       X| $FEE2

shot06:
       .byte $01 ; |       X| $FE7A
       .byte $07 ; |     XXX| $FE7C
       .byte $07 ; |     XXX| $FE7C
       .byte $0F ; |    XXXX| $FE7D
       .byte $0F ; |    XXXX| $FE7E
       .byte $07 ; |     XXX| $FE7F
       .byte $07 ; |     XXX| $FE7C
       .byte $01 ; |       X| $FE81

shot05:
       .byte $01 ; |       X| $FE6E
       .byte $03 ; |      XX| $FE6F
       .byte $07 ; |     XXX| $FE70
       .byte $07 ; |     XXX| $FE71
       .byte $03 ; |      XX| $FE72
       .byte $01 ; |       X| $FE73

shot04:
       .byte $01 ; |       X| $FE64
       .byte $03 ; |      XX| $FE65
       .byte $03 ; |      XX| $FE66
       .byte $01 ; |       X| $FE67

shot03:
       .byte $01 ; |       X| $FE5B
       .byte $03 ; |      XX| $FE5C
       .byte $01 ; |       X| $FE5D

shot02:
       .byte $01 ; |       X| $FE4A
       .byte $01 ; |       X| $FE4B

shot01:
       .byte $01 ; |       X| $FE4B

mothership01:
       .byte %00011111
       .byte %11111111
       .byte %11111111
       .byte %01111111
       .byte %01111000
       .byte %01110000
       .byte %00100000

mothership02:
       .byte %00000000
       .byte %00001111
       .byte %01111111
       .byte %00000111
       .byte %00000000
       .byte %00000000
       .byte %00000000

void:
       .byte %00000000
       .byte %00000000
       .byte %00000000
       .byte %00000000
       .byte %00000000

laser03:
       .byte %00001110
       .byte %00011111
       .byte %00011111
       .byte %00011111
       .byte %00001110

laser02:
       .byte %00000110
       .byte %00001111
       .byte %00001111
       .byte %00000110

laser01:
       .byte %00000110
       .byte %00001111
       .byte %00000110

explosion08
       .byte %00000001
       .byte %00010000
       .byte %00100100
       .byte %11000010
       .byte %01001111
       .byte %10111101
       .byte %00001010
       .byte %10010010
       .byte %00001010
       .byte %00010001
       .byte %00000011
       .byte %00100000
       .byte %00001001

explosion06
       .byte %00000110
       .byte %00001001
       .byte %00010000
       .byte %00000111
       .byte %00100000
       .byte %00001010
       .byte %00010100
       .byte %00010101
       .byte %00001001
       .byte %00000001

explosion04
       .byte %00000010
       .byte %00000100
       .byte %00001101
       .byte %00000101
       .byte %00000010
       .byte %00000001

explosion02
       .byte %00000001
       .byte %00000010
       .byte %00000001

explosion208
       .byte %00000001
       .byte %00000100
       .byte %00010010
       .byte %00110100
       .byte %00101100
       .byte %00000010
       .byte %00010110
       .byte %00001001
       .byte %00000100
       .byte %01010111
       .byte %00010000
       .byte %00010011
       .byte %00001101

explosion206
       .byte %00000101
       .byte %00000011
       .byte %00001001
       .byte %00100101
       .byte %00100110
       .byte %00100010
       .byte %00001010
       .byte %00010100
       .byte %00000011
       .byte %00000101

explosion204
       .byte %00000010
       .byte %00000101
       .byte %00000110
       .byte %00001100
       .byte %00000101
       .byte %00000010

explosion202
       .byte %00000000
       .byte %00000011
       .byte %00000001

    ORG     $1D00
    RORG    $DD00

starenamtab
    .byte $00,$00,$00,$00,$00,$02,$00,$00,$00,$00,$00,$02,$00,$00,$00,$00
    .byte $00,$02,$00,$00,$00,$02,$00,$00,$02,$00,$00,$02,$00,$00,$00,$02
    .byte $02,$00,$02,$00,$00,$00,$02,$00,$00,$00,$00,$00,$02,$00,$00,$00
    .byte $02,$00,$00,$02,$00,$00,$00,$02,$00,$00,$00,$00,$02,$00,$02,$00
    .byte $00,$00,$00,$02,$00,$00,$02,$00,$00,$00,$02,$00,$00,$02,$00,$00
    .byte $00,$02,$00,$00,$00,$02,$00,$00,$02,$00,$00,$02,$00,$00,$00,$02
    .byte $02,$00,$02,$00,$00,$00,$02,$00,$00,$00,$00,$00,$02,$00,$00,$00
    .byte $02,$00,$00,$02,$00,$00,$00,$02,$00,$00,$00,$00,$02,$00,$02,$00
    .byte $00,$00,$00,$00,$00,$02,$00,$00,$00,$00,$00,$02,$00,$00,$00,$00
    .byte $00,$02,$00,$00,$00,$02,$00,$00,$02,$00,$00,$02,$00,$00,$00,$02
    .byte $02,$00,$02,$00,$00,$00,$02,$00,$00,$00,$00,$00,$02,$00,$00,$00
    .byte $02,$00,$00,$02,$00,$00,$00,$02,$00,$00,$00,$00,$02,$00,$02,$00
    .byte $00,$00,$00,$02,$00,$00,$02,$00,$00,$00,$02,$00,$00,$02,$00,$00
    .byte $00,$02,$00,$00,$00,$02,$00,$00,$02,$00,$00,$02,$00,$00,$00,$02
    .byte $02,$00,$02,$00,$00,$00,$02,$00,$00,$00,$00,$00,$02,$00,$00,$00
    .byte $02,$00,$00,$02,$00,$00,$00,$02,$00,$00,$00,$00,$02,$00,$02,$00

movetab1
    .byte                     $12,$02,$F2,$E2,$D2,$C2,$B2,$A2
    .byte $63,$53,$43,$33,$23,$13,$03,$F3,$E3,$D3,$C3,$B3,$A3,$93,$83
    .byte $64,$54,$44,$34,$24,$14,$04,$F4,$E4,$D4,$C4,$B4,$A4,$94,$84
    .byte $65,$55,$45,$35,$25,$15,$05,$F5,$E5,$D5,$C5,$B5,$A5,$95,$85
    .byte $66,$56,$46,$36,$26,$16,$06,$F6,$E6,$D6,$C6,$B6,$A6,$96,$86
    .byte $67,$57,$47,$37,$27,$17,$07,$F7,$E7,$D7,$C7,$B7,$A7,$97,$87
    .byte $68,$58,$48,$38,$28,$18,$08,$F8,$E8,$D8,$C8,$B8,$A8,$98,$88
    .byte $69,$59,$49,$39,$29,$19,$09,$F9,$E9,$D9,$C9,$B9,$A9,$99,$89
    .byte $6A,$5A,$4A,$3A,$2A,$1A,$0A,$FA,$EA,$DA,$CA,$BA,$AA,$9A,$8A

movetab2
    .byte                             $02,$F2,$E2,$D2,$C2,$B2,$A2,$92
movetab3
    .byte $73,$63,$53,$43,$33,$23,$13,$03,$F3,$E3,$D3,$C3,$B3,$A3,$93
    .byte $74,$64,$54,$44,$34,$24,$14,$04,$F4,$E4,$D4,$C4,$B4,$A4,$94
    .byte $75,$65,$55,$45,$35,$25,$15,$05,$F5,$E5,$D5,$C5,$B5,$A5,$95
    .byte $76,$66,$56,$46,$36,$26,$16,$06,$F6,$E6,$D6,$C6,$B6,$A6,$96
    .byte $77,$67,$57,$47,$37,$27,$17,$07,$F7,$E7,$D7,$C7,$B7,$A7,$97
    .byte $78,$68,$58,$48,$38,$28,$18,$08,$F8,$E8,$D8,$C8,$B8,$A8,$98
    .byte $79,$69,$59,$49,$39,$29,$19,$09,$F9,$E9,$D9,$C9,$B9,$A9,$99
    .byte $7A,$6A,$5A,$4A,$3A,$2A,$1A,$0A,$FA,$EA,$DA,$CA,$BA,$AA,$9A
    .byte $7B,$6B,$5B,$4B,$3B,$2B,$1B,$0B,$FB,$EB,$DB,$CB,$BB,$AB,$9B
    .byte $7C,$6C,$5C,$4C,$3C,$2C,$1C,$0C,$FC,$EC,$DC,$CC,$BC,$AC,$9C
    .byte $7D,$6D,$5D,$4D,$3D,$2D,$1D,$0D,$FD,$ED

movetab4
    .byte $70,$60,$50,$40,$30,$20,$10,$00,$F0,$E0,$D0,$C0,$B0,$A0,$90
    .byte $71,$61,$51,$41,$31,$21,$11,$01,$F1,$E1,$D1,$C1,$B1,$A1,$91
    .byte $72,$62

isprypos
    .byte $00,$00,$39,$39,$50,$60,$70,$80,$12,$20
isprtsizetype
    .byte $01,$28,$29,$0C,$2A,$2B,$2C,$2D,$2E,$2F
isprxpos
    .byte $C0,$00,$26,$56,$A0,$C0,$E0,$00,$20,$40
iymov
    .byte $00,$00,$FF,$00,$01,$FF,$00,$01,$FF,$01
ixmov
    .byte $00,$02,$FF,$01,$FF,$01,$FF,$01,$00,$01

shotypos
    .byte $09,$09,$3F,$3F

shotxpos
    .byte $6E,$0E,$0E,$6E

shotymov
    .byte $03,$03,$FD,$FD

shotxmov
    .byte $FA,$06,$06,$FA

shottab .byte %00000000, %00000100, %00001000
        .byte %00001000, %00000100, %00000000

radarsprite
    .byte #$00
    .byte #$00
    .byte #$00
    .byte #$00
    .byte #$00
    .byte #$00
    .byte #$00
    .byte #$00
    .byte #$FF
    .byte #$FF
    .byte #$FF
    .byte #$FF
    .byte #$FF
    .byte #$FF
    .byte #$FF
    .byte #$FF
    .byte #$FF
    .byte #$FF
    .byte #$FF
    .byte #$FF
    .byte #$FF
    .byte #$FF
    .byte #$FF
    .byte #$FF
    .byte #$00
    .byte #$00
    .byte #$00
    .byte #$00
    .byte #$00
    .byte #$00
    .byte #$00
    .byte #$00

shieldpattern1
    .byte #%11111111
    .byte #%11111111
    .byte #%11111111
    .byte #%11111111
    .byte #%11111111
    .byte #%11111111
    .byte #%11111111
    .byte #%11111111
    .byte #%11111111
    .byte #%11111111
    .byte #%11111111
    .byte #%11111111
    .byte #%11111111
    .byte #%11111111
    .byte #%11111111
    .byte #%11111110
    .byte #%11111110
    .byte #%11111110

shieldpattern2
    .byte #%11111111
    .byte #%11111111
    .byte #%11111111
    .byte #%01111111
    .byte #%01111111
    .byte #%01111111
    .byte #%01011111
    .byte #%01011111
    .byte #%01011111
    .byte #%01010111
    .byte #%01010111
    .byte #%01010111
    .byte #%01010101
    .byte #%01010101
    .byte #%01010101
    .byte #%01010101
    .byte #%01010101
    .byte #%01010101

shieldor
    .byte #%11111111
    .byte #%00111111
    .byte #%00001111
    .byte #%00000011
    .byte #%00000000
    .byte #%00000000

shieldloop
    .byte #$03
    .byte #$06
    .byte #$09
    .byte #$0C
    .byte #$0F
    .byte #$12

shieldcolor
    .byte #$00  ;Dummy
    .byte #$0A
    .byte #$08
    .byte #$06
    .byte #$04
    .byte #$02

    ORG     $1FF0
    RORG    $DFF0

SwitchToBank1:
    BIT $1FF9
    JMP MainGame

    .byte   $FF, $FF, $FF, $FF
    .word   SwitchToBank1
    .word   SwitchToBank1
    .word   SwitchToBank1

;----------------------------------------------------------
; 4K INTRO
;----------------------------------------------------------

INTROLINES  = 10
STARLETTERS =  5
NUMLETTERS  =  9

;===============================================================================
; Z P - V A R I A B L E S
;===============================================================================

    SEG.U   variables
    ORG     $80

; Both Banks:
gameState       ds 1                ; X/X/X/X/X/X/X/INTRO
bcdScore        ds 3                ; Last/Current score
bcdWave         ds 1                ; desired/current wave
frameCounter    ds 1                ; Incremented every frame

; This Bank:
tmpVar      .byte
tmpVar2     .byte
tmpVar3     .byte
lines       .byte
logoPtr     .word
curRow      .byte               ; 0..9
curCol      .byte               ; 0..5
curDir      .byte               ; 0..7
curBit      .byte               ; 0..7
curLetter   .byte               ; 0..9
logoCol     .byte
logo        ds 6*INTROLINES
charPtr     ds.w 6
soundCount  ds 1 ; current note to be played
decayCount  ds 1 ; note decay

;===============================================================================
; R O M - C O D E
;===============================================================================

    SEG     Bank1
    ORG     $2000
    RORG    $F000

Pot2Tab:
    .byte   $80, $40, $20, $10, $08, $04, $02, $01

MainLoop:
    jsr     VerticalBlank
    jsr     GameCalc
    jsr     DrawScreen
    JMP     OverScanIntro

VerticalBlank SUBROUTINE
    lda     #2
    sta     WSYNC
    sta     VSYNC
    sta     WSYNC
    sta     WSYNC
    lsr
    ldx     #44
    sta     WSYNC
    sta     VSYNC
    stx     TIM64T
    rts

Draw48Pixel SUBROUTINE
    stx     lines

    ldx     #11
.loopSet:
    lda     PatternTab,y
    sta     charPtr,x
    dey
    dex
    bpl     .loopSet

Draw48Pixel2

; slightly modified 48 pixel version:
loop2
    lda     lines           ;+3  63  189
    lsr                     ; 2
    tay                     ; 2
    lda     (charPtr),y     ;+5  68  204
    sta     WSYNC           ;go
    sta.w   GRP0            ;+3  71  213      D1     --      --     --
    lda     (charPtr+$2),y  ;+5   5   15
    sta     GRP1            ;+3   8   24      D1     D1      D2     --
    lda     (charPtr+$4),y  ;+5  13   39
    sta     GRP0            ;+3  16   48      D3     D1      D2     D2
    lda     (charPtr+$6),y  ;+5  21   63
    sta     tmpVar2         ;+3  24   72
    lax     (charPtr+$8),y  ;+5  29   87
    lda     (charPtr+$A),y  ;+5  36  108
    ldy     tmpVar2         ;+3  41  123              !
    sty     GRP1            ;+3  44  132      D3     D3      D4     D2!
    stx     GRP0            ;+3  47  141      D5     D3!     D4     D4
    sta     GRP1            ;+3  50  150      D5     D5      D6     D4!
    sta     GRP0            ;+3  53  159      D4*    D5!     D6     D6
    dec     lines           ;+5  58  174                             !
    bpl     loop2           ;+2  60  180

    ldy     #0
    sty     GRP0
    sty     GRP1
    sty     GRP0
    rts


NextLetter SUBROUTINE
    ldy     curLetter
    iny
    cpy     #NUMLETTERS+1
    bcc     .skipRestart
    ldy     #0
StartLetter:
    lda     #$42
    sta     logoCol
    lda     #<Star0
    beq     .setPtr

.skipRestart:
    cpy     #STARLETTERS
    bne     .skipFire
    lda     #<Fire0
.setPtr:
    sta     logoPtr
    ldx     #INTROLINES*6-1
.loopClear:
    lda     #0
    sta     logo,x
    dex
    bpl     .loopClear
.skipFire:
    sty     curLetter
    lda     ColTab,y
    sta     curCol
    lda     RowTab,y
    sta     curRow
    lda     BitTab,y
    sta     curBit
    lda     #0
    sta     curDir
RTS:
    rts

GameCalc SUBROUTINE

    SED
    CLC
    LDA bcdScore+2
    ADC #$01
    STA bcdScore+2
    LDA bcdScore+1
    ADC #$00
    STA bcdScore+1
    LDA bcdScore
    ADC #$00
    STA bcdScore

    LDA frameCounter
    AND #$0F
    BNE WaveOk

    LDA SWCHA
    ASL
    TAY
    BCS .RightDone
    LDA bcdWave
    ADC #$05
    STA bcdWave
.RightDone
    TYA
    ASL
    BCS WaveOk
    LDA bcdWave
    SEC
    SBC #$05
    STA bcdWave

WaveOk
    CLD

    dec frameCounter
    lda     frameCounter
    and     #$01
    bne     RTS

    lda     frameCounter
    ldy     curLetter
    cpy     #NUMLETTERS
    bne     .cont
    ldx     #$44
    stx     logoCol
    tax
    bne     RTS
    lda     #$00
    sta     frameCounter
    jmp     NextLetter

.cont:
    ldy     curCol
    lda     MultTab,y
    clc
    adc     curRow
    tax
    lda     logo,x
    ldy     curBit
    ora     Pot2Tab,y
    sta     logo,x

scanRow     = tmpVar
scanCol     = tmpVar+1
scanIdx     = lines

    ldx     #0
    lda     curDir
    lsr                         ;       odd or even old direction?
    bcc     .even
    ldx     #ScanTabOdd-ScanTab
.even:
    stx     scanIdx

.loopScan:
    ldy     scanIdx
    inc     scanIdx
    lda     curDir
    clc
    adc     ScanTab,y
    and     #$07
    sta     curDir
    tay

    lda     curRow
    clc
    adc     DirYTab,y
    cmp     #INTROLINES
    bcs     .loopScan
    sta     scanRow

    lda     curBit
    clc
    adc     DirXTab,y
    ldy     curCol
    tax
    cmp     #8
    bcc     .sameByte
    beq     .addLines
    lda     #-2
    .byte $2c    ;       skips lda #0
.addLines:
    lda     #0
    adc     curCol
    cmp     #6
    bcs     .loopScan
    tay
    txa
    and     #$07
    tax
.sameByte:
    sty     scanCol
    lda     MultTab,y
    clc
    adc     scanRow
    tay
    lda     (logoPtr),y
    and     Pot2Tab,x
    beq     .loopScan
    stx     curBit
    lda     scanRow
    sta     curRow
    lda     scanCol
    sta     curCol
    ldy     curLetter
    lda     curCol
    cmp     ColTab,y
    bne     .skipNext
    txa
    cmp     BitTab,y
    bne     .skipNext
    lda     curRow
    cmp     RowTab,y
    bne     .skipNext
    jsr     NextLetter
.skipNext:
    rts

DrawScreen SUBROUTINE
    tsx
    stx     tmpVar

    ldx     #227
.waitTim:
    lda     INTIM
    bne     .waitTim

    sta     WSYNC
    sta     VBLANK
    stx     TIM64T
;---------------------------------------------------------------

    LDX #14
    JSR BurnXLines

    LDA gameState
    BNE ContinueIntro2
    JMP DrawScore

ContinueIntro2
    ldx     #$44
    stx     COLUP0
    stx     COLUP1

    ldx     #7*2-1
    ldy     #12*3-1
    jsr     Draw48Pixel     ;   Xype

    lda     #8
    ldx     curLetter
    cpx     #1
    bcs     .colorOk

    lda     frameCounter
    eor     #$FF
    lsr
    lsr
    and     #$0f
    cmp     #$08
    bcc     .colorOk
    eor     #$0f
.colorOk:
    sta     COLUP0
    sta     COLUP1

    LDX #4
    JSR BurnXLines

    ldx     #7*2-1
    ldy     #12*4-1
    jsr     Draw48Pixel     ;   presents

    LDX #12
    JSR BurnXLines

    lda     logoCol
    sta     COLUP0
    sta     COLUP1

    LDA #45
    STA tmpVar3

    ldy     curLetter
    cpy     #STARLETTERS
    bcc     .skipStar

    LDA #21
    STA tmpVar3
    ldx     #INTROLINES*2-1
    ldy     #12*1-1
    jsr     Draw48Pixel

.skipStar:
    sta     WSYNC
;---------------------------------------------------------------

    ldx     #INTROLINES*2-1
    ldy     #12*5-1
    jsr     Draw48Pixel

    LDX tmpVar3
    JSR BurnXLines

ContinueScore

    LDA #8
    STA COLUP0
    STA COLUP1

    LDA bcdWave
    AND #$0F
    BEQ DoMeteors
    CMP #$05
    BEQ DoMeteors

    LDX #11
    LDA bcdWave         ; load low/high values of states
    AND #$0F            ; select lower nibble
    TAY                 ; X-> shape of value
    LDA #>zero          ; Load pointer to desired value
    STA charPtr,X       ; Store sprite pointer
    STA charPtr-2,X       ; Store sprite pointer
    DEX
    LDA digittab,Y      ; Load pointer to desired value
    STA charPtr,X       ; Store sprite pointer
    DEX
    DEX 
    LDA bcdWave         ; load low/high values of states
    LSR                 ; select higher nibble... 
    LSR                 ; ...of status value (bcd!)
    LSR                 ; (numbers 0-9)
    LSR                 ;
    TAY                 ; X-> shape of value
    LDA digittab,Y      ; Load pointer to desired value
    STA charPtr,X       ; Store sprite pointer
    DEX
    LDY #12*8-5

.loopSet:
    lda     PatternTab,y
    sta     charPtr,x
    dey
    dex
    bpl     .loopSet

    LDX #7*2-1
    STX lines
    jsr Draw48Pixel2    ; Wave:
    JMP MeteorsDone

DoMeteors
    ldx #7*2-1
    ldy #12*7-1
    jsr Draw48Pixel         ; Meteors!

MeteorsDone
    ldx     #7*2-1
    ldy     #12*6-1
    jsr     Draw48Pixel     ;   Score:

    LDX #11
NextDigit
    TXA
    LSR
    LSR
    TAY
    LDA bcdScore,Y      ; load low/high values of states
    STA tmpVar3
    AND #$0F            ; select lower nibble
    TAY                 ; X-> shape of value
    LDA #>zero          ; Load pointer to desired value
    STA charPtr,X       ; Store sprite pointer
    STA charPtr-2,X       ; Store sprite pointer
    DEX
    LDA digittab,Y      ; Load pointer to desired value
    STA charPtr,X       ; Store sprite pointer
    DEX
    DEX 
    LDA tmpVar3         ; load low/high values of states
    LSR                 ; select higher nibble... 
    LSR                 ; ...of status value (bcd!)
    LSR                 ; (numbers 0-9)
    LSR                 ;
    TAY                 ; X-> shape of value
    LDA digittab,Y      ; Load pointer to desired value
    STA charPtr,X       ; Store sprite pointer
    DEX
    BPL NextDigit
    LDX #7*2-1
    STX lines
    jsr Draw48Pixel2    ;   Score

WaitScreen
    ldx     #2
    lda     INTIM
    bne     WaitScreen
    sta     WSYNC
    stx     VBLANK
    ldx     tmpVar
    txs
    rts

DrawScore
    LDA #$42
    STA COLUP0
    STA COLUP1
    ldx #INTROLINES*2-1
    ldy #12*1-1
    jsr Draw48Pixel         ; Star

    sta WSYNC
    ldx #INTROLINES*2-1
    ldy #12*2-1
    jsr Draw48Pixel         ; Fire

    LDX #20
    JSR BurnXLines

    JMP ContinueScore

OverScanIntro SUBROUTINE
    LDA #$25
    STA TIM64T          ; 30 lines overscan

;-----------------------------------------------------------
; BANKSWITCH 
;-----------------------------------------------------------

    LDA INPT4
    BMI ContinueIntro
    JMP SwitchToBank0

ContinueIntro
    LSR SWCHB
    BCS ContinueIntro3
    JMP StartReset

ContinueIntro3
    LDA gameState
    LSR
    BCC NoMusicNow
    JSR PlayCash

NoMusicNow
    JMP OverscanLoop2

start
    CLD
    LDA #$00
    TAX

ZeroZero
    STA VSYNC,X
    TXS
    INX
    BNE ZeroZero

    LDA #$30        ; No timer is set on first start
    STA TIM64T

StartReset
    LDX #$00
    STX soundCount  ; music on
    INX
    STX gameState   
    STX bcdWave
    INX
    STX decayCount
    JMP .Continue

StartScore
    LDA gameState
    BNE StartReset

.Continue
    LDY #$00        ; minimal reset
    STY COLUPF
    STY CTRLPF
    STY REFP1
    STY GRP0

    jsr StartLetter

    lda     #$00
    sta     frameCounter

    lda     #>Star0
    sta     logoPtr+1

    lda     #%01011
    sta     NUSIZ0
    sta     NUSIZ1
    sta     VDELP0
    sta     VDELP1

    sta     WSYNC
    ldx     #7

.wait:
    dex
    bne     .wait
    sta.w   RESP0       ;
    sta     RESP1
    lda     #$14
    sta     HMP1
    lsr
    sta     HMP0
    sta     WSYNC
    sta     HMOVE

OverscanLoop2
    lda     INTIM
    bne     OverscanLoop2
    JMP MainLoop

BurnXLines
    STA WSYNC
    DEX
    BNE BurnXLines
    RTS

PlayCash
            DEC decayCount
            DEC decayCount
            BNE DecayNotZero
            LDX soundCount
            LDA soundfreq,X
            AND #$0F
            BEQ NoNote0
            TAY
            LDA voice1disttab,Y
            STA AUDC0
            LDA voice1freqtab,Y
            STA AUDF0
NoNote0
            LDX soundCount
            LDA soundfreq,X
            AND #$F0
            BEQ NoNote1
            LSR
            LSR
            LSR
            LSR
            TAY
            LDA voice2disttab,Y
            STA AUDC1
            LDA voice2freqtab,Y
            STA AUDF1
NoNote1
            LDA #14
            STA AUDV0
            LDA #6
            STA AUDV1
            LDA #16
            STA decayCount
            INC soundCount
            RTS

DecayNotZero
            LDX soundCount
            LDA soundfreq,X
            AND #$0F
            BEQ NoDecay0
            LDA decayCount
            STA AUDV0
NoDecay0
            LDX soundCount
            LDA soundfreq,X
            AND #$F0
            BEQ NoDecay1
            LDA decayCount
            LSR
            STA AUDV1
NoDecay1
            RTS

;Sound equates for 16 different states/notes voice 1

HOLD1       = $00   ; Hold last note
SIL1        = $01   ; Silencium please

E61         = $02   ; note E6 
D61         = $03   ; note D6 
C61         = $04   ; note C6 
                            
H51         = $05   ; note H5 
A51         = $06   ; note A5 

G51         = $07   ; note G5 
F551        = $08   ; note F#5
E51         = $09   ; note E5 
D51         = $0A   ; note D5 
C51         = $0B   ; note C5 

H41         = $0C   ; note H4 
A41         = $0D   ; note A4 
G41         = $0E   ; note G4 
F451        = $0F   ; note F#4

;Sound equates for 10 different states/notes voice 2

HOLD2       = $00   ; Hold last note
DRM2        = $10   ; Base Drum :-)
A32         = $20   ; note A3
G32         = $30   ; note G3 
F352        = $40   ; note F#3
E32         = $50   ; note E3
D32         = $60   ; note D3 
C32         = $70   ; note C3 

H22         = $80   ; note H2
A22         = $90   ; note A2
G22         = $A0   ; note G2

; frequency & distortion tabs - notes see above!

soundfreq
   .byte #G32   | #D61
   .byte #HOLD2 | #SIL1
   .byte #DRM2  | #D61
   .byte #HOLD2 | #D61
   .byte #G32   | #SIL1
   .byte #HOLD2 | #D61
   .byte #DRM2  | #D61
   .byte #HOLD2 | #HOLD1
   .byte #C32   | #E61
   .byte #HOLD2 | #HOLD1
   .byte #DRM2  | #C61
   .byte #HOLD2 | #HOLD1
   .byte #G32   | #D61
   .byte #HOLD2 | #HOLD1
   .byte #DRM2  | #HOLD1
   .byte #HOLD2 | #HOLD1
   .byte #D32   | #HOLD1
   .byte #HOLD2 | #HOLD1
   .byte #DRM2  | #HOLD1
   .byte #HOLD2 | #HOLD1
   .byte #G32   | #HOLD1
   .byte #HOLD2 | #HOLD1
   .byte #DRM2  | #HOLD1
   .byte #HOLD2 | #HOLD1
   .byte #D32   | #HOLD1
   .byte #HOLD2 | #HOLD1
   .byte #DRM2  | #HOLD1
   .byte #HOLD2 | #HOLD1
   .byte #G32   | #H51
   .byte #HOLD2 | #SIL1
   .byte #DRM2  | #H51
   .byte #HOLD2 | #H51
   .byte #D32   | #SIL1
   .byte #HOLD2 | #H51
   .byte #DRM2  | #H51
   .byte #HOLD2 | #HOLD1
   .byte #D32   | #C61
   .byte #HOLD2 | #HOLD1
   .byte #DRM2  | #A51
   .byte #HOLD2 | #HOLD1
   .byte #G32   | #H51
   .byte #HOLD2 | #HOLD1
   .byte #DRM2  | #HOLD1
   .byte #HOLD2 | #HOLD1
   .byte #D32   | #HOLD1
   .byte #HOLD2 | #HOLD1
   .byte #DRM2  | #HOLD1
   .byte #HOLD2 | #HOLD1
   .byte #G32   | #HOLD1
   .byte #HOLD2 | #HOLD1
   .byte #DRM2  | #HOLD1
   .byte #HOLD2 | #HOLD1
   .byte #D32   | #HOLD1
   .byte #HOLD2 | #HOLD1
   .byte #E32   | #HOLD1
   .byte #HOLD2 | #HOLD1
   .byte #G32   | #SIL1
   .byte #HOLD2 | #HOLD1
   .byte #D32   | #HOLD1
   .byte #HOLD2 | #HOLD1
   .byte #E32   | #HOLD1
   .byte #HOLD2 | #HOLD1
   .byte #F352  | #HOLD1
   .byte #HOLD2 | #HOLD1
   .byte #G32   | #D51
   .byte #HOLD2 | #HOLD1
   .byte #DRM2  | #HOLD1
   .byte #HOLD2 | #HOLD1
   .byte #D32   | #SIL1
   .byte #HOLD2 | #HOLD1
   .byte #DRM2  | #HOLD1
   .byte #HOLD2 | #HOLD1
   .byte #G32   | #HOLD1
   .byte #HOLD2 | #HOLD1
   .byte #DRM2  | #D51
   .byte #HOLD2 | #HOLD1
   .byte #C32   | #E51
   .byte #HOLD2 | #HOLD1
   .byte #DRM2  | #C51
   .byte #HOLD2 | #HOLD1
   .byte #G32   | #D51
   .byte #HOLD2 | #HOLD1
   .byte #DRM2  | #D61
   .byte #HOLD2 | #D61
   .byte #D32   | #SIL1
   .byte #HOLD2 | #D61
   .byte #DRM2  | #D61
   .byte #HOLD2 | #HOLD1
   .byte #C32   | #E61
   .byte #HOLD2 | #HOLD1
   .byte #DRM2  | #C61
   .byte #HOLD2 | #HOLD1
   .byte #G32   | #D61
   .byte #HOLD2 | #HOLD1
   .byte #DRM2  | #HOLD1
   .byte #HOLD2 | #HOLD1
   .byte #D32   | #HOLD1
   .byte #HOLD2 | #HOLD1
   .byte #DRM2  | #HOLD1
   .byte #HOLD2 | #HOLD1
   .byte #G32   | #HOLD1
   .byte #HOLD2 | #HOLD1
   .byte #DRM2  | #HOLD1
   .byte #HOLD2 | #HOLD1
   .byte #D32   | #HOLD1
   .byte #HOLD2 | #HOLD1
   .byte #DRM2  | #HOLD1
   .byte #HOLD2 | #HOLD1
   .byte #G32   | #H41
   .byte #HOLD2 | #HOLD1
   .byte #DRM2  | #H41
   .byte #HOLD2 | #H41
   .byte #D32   | #HOLD1
   .byte #HOLD2 | #SIL1
   .byte #DRM2  | #H41
   .byte #HOLD2 | #H41
   .byte #D32   | #A41
   .byte #HOLD2 | #HOLD1
   .byte #DRM2  | #F451
   .byte #HOLD2 | #HOLD1
   .byte #G32   | #G41
   .byte #HOLD2 | #HOLD1
   .byte #DRM2  | #HOLD1
   .byte #HOLD2 | #HOLD1
   .byte #D32   | #SIL1
   .byte #HOLD2 | #HOLD1
   .byte #DRM2  | #HOLD1
   .byte #HOLD2 | #HOLD1
   .byte #G32   | #HOLD1
   .byte #HOLD2 | #HOLD1
   .byte #DRM2  | #HOLD1
   .byte #HOLD2 | #HOLD1
   .byte #H22   | #HOLD1
   .byte #HOLD2 | #HOLD1
   .byte #C32   | #HOLD1
   .byte #HOLD2 | #HOLD1
   .byte #D32   | #D51
   .byte #HOLD2 | #HOLD1
   .byte #DRM2  | #HOLD1
   .byte #HOLD2 | #SIL1
   .byte #A22   | #F551
   .byte #HOLD2 | #HOLD1
   .byte #DRM2  | #HOLD1
   .byte #HOLD2 | #SIL1
   .byte #D32   | #A51
   .byte #HOLD2 | #HOLD1
   .byte #DRM2  | #HOLD1
   .byte #HOLD2 | #SIL1
   .byte #A22   | #A51
   .byte #HOLD2 | #A51
   .byte #DRM2  | #HOLD1
   .byte #HOLD2 | #HOLD1
   .byte #C32   | #G51
   .byte #HOLD2 | #G51
   .byte #DRM2  | #HOLD1
   .byte #HOLD2 | #HOLD1
   .byte #G32   | #G51
   .byte #HOLD2 | #HOLD1
   .byte #E32   | #G51
   .byte #HOLD2 | #HOLD1
   .byte #G22   | #E51
   .byte #HOLD2 | #D51
   .byte #DRM2  | #HOLD1
   .byte #HOLD2 | #HOLD1
   .byte #H22   | #SIL1
   .byte #HOLD2 | #H41
   .byte #C32   | #C51
   .byte #HOLD2 | #HOLD1
   .byte #D32   | #D51
   .byte #HOLD2 | #HOLD1
   .byte #DRM2  | #HOLD1
   .byte #HOLD2 | #SIL1
   .byte #A22   | #F551
   .byte #HOLD2 | #HOLD1
   .byte #DRM2  | #HOLD1
   .byte #HOLD2 | #SIL1
   .byte #D32   | #A51
   .byte #HOLD2 | #HOLD1
   .byte #DRM2  | #HOLD1
   .byte #HOLD2 | #SIL1
   .byte #A22   | #HOLD1
   .byte #HOLD2 | #F551
   .byte #DRM2  | #F551
   .byte #HOLD2 | #HOLD1
   .byte #C32   | #G51
   .byte #HOLD2 | #HOLD1
   .byte #A32   | #HOLD1
   .byte #HOLD2 | #SIL1
   .byte #G32   | #G51
   .byte #HOLD2 | #HOLD1
   .byte #E32   | #HOLD1
   .byte #HOLD2 | #HOLD1
   .byte #G22   | #E51
   .byte #HOLD2 | #HOLD1
   .byte #DRM2  | #D51
   .byte #HOLD2 | #HOLD1
   .byte #C32   | #SIL1
   .byte #HOLD2 | #H41
   .byte #D32   | #H41
   .byte #HOLD2 | #SIL1
   .byte #G32   | #G41
   .byte #HOLD2 | #HOLD1
   .byte #DRM2  | #HOLD1
   .byte #HOLD2 | #SIL1
   .byte #D32   | #H41
   .byte #HOLD2 | #HOLD1
   .byte #DRM2  | #HOLD1
   .byte #HOLD2 | #SIL1
   .byte #G32   | #D51
   .byte #HOLD2 | #HOLD1
   .byte #DRM2  | #HOLD1
   .byte #HOLD2 | #HOLD1
   .byte #D32   | #SIL1
   .byte #HOLD2 | #HOLD1
   .byte #DRM2  | #HOLD1
   .byte #HOLD2 | #HOLD1
   .byte #G22   | #HOLD1
   .byte #HOLD2 | #HOLD1
   .byte #DRM2  | #H41
   .byte #HOLD2 | #HOLD1
   .byte #D32   | #C51
   .byte #HOLD2 | #HOLD1
   .byte #DRM2  | #A41
   .byte #HOLD2 | #HOLD1
   .byte #G32   | #H41
   .byte #HOLD2 | #HOLD1
   .byte #DRM2  | #D51
   .byte #HOLD2 | #HOLD1
   .byte #D32   | #HOLD1
   .byte #HOLD2 | #HOLD1
   .byte #DRM2  | #SIL1
   .byte #HOLD2 | #HOLD1
   .byte #G32   | #HOLD1
   .byte #HOLD2 | #HOLD1
   .byte #DRM2  | #G41
   .byte #HOLD2 | #HOLD1
   .byte #D32   | #A41
   .byte #HOLD2 | #HOLD1
   .byte #DRM2  | #F451
   .byte #HOLD2 | #HOLD1
   .byte #G32   | #G41
   .byte #HOLD2 | #HOLD1
   .byte #DRM2  | #HOLD1
   .byte #HOLD2 | #HOLD1
   .byte #D32   | #SIL1
   .byte #HOLD2 | #HOLD1
   .byte #DRM2  | #HOLD1
   .byte #HOLD2 | #HOLD1
   .byte #G32   | #HOLD1
   .byte #HOLD2 | #HOLD1
   .byte #D32   | #HOLD1
   .byte #HOLD2 | #HOLD1
   .byte #E32   | #HOLD1
   .byte #HOLD2 | #HOLD1
   .byte #F352  | #HOLD1
   .byte #HOLD2 | #HOLD1

voice2freqtab
    .byte #00   ; Dummy    
    .byte #07   ; 
    .byte #18   ; 
    .byte #20   ; 
    .byte #10   ; 
    .byte #24   ; 
    .byte #13   ; 
    .byte #31   ; 
    .byte #15   ; 
    .byte #17   ; 
    .byte #20   ; 

voice2disttab
    .byte #00   ; Dummy    
    .byte #$08  ; 
    .byte #01   ; 
    .byte #01   ; 
    .byte #07   ; 
    .byte #01   ; 
    .byte #07   ; 
    .byte #01   ; 
    .byte #07   ; 
    .byte #07   ; 
    .byte #07   ; 

voice1disttab
    .byte #00   ; Dummy    
    .byte #04   ; 
    .byte #04   ; 
    .byte #04   ; 
    .byte #04   ; 
    .byte #04   ; 
    .byte #04   ; 
    .byte #04   ; 
    .byte #04   ; 
    .byte #04   ; 
    .byte #04   ; 
    .byte #04   ; 
    .byte #04   ; 
    .byte #12   ; 
    .byte #12   ; 
    .byte #12   ; 

voice1freqtab
    .byte #00   ; Dummy    
    .byte #00   ; 
    .byte #11   ; 
    .byte #12   ; 
    .byte #14   ; 
    .byte #15   ; 
    .byte #17   ; 
    .byte #19   ; 
    .byte #20   ; 
    .byte #23   ; 
    .byte #26   ; 
    .byte #29   ; 
    .byte #31   ; 
    .byte #11   ; 
    .byte #12   ; 
    .byte #13   ; 

;===============================================================================
; R O M - T A B L E S (Bank 0)
;===============================================================================

    ORG     $2500
    RORG    $F500

Star0:
    .byte   %11111111
    .byte   %10000000
    .byte   %10000000
    .byte   %11111111
    .byte   %00000001
    .byte   %00000010
    .byte   %00000010
    .byte   %00000010
    .byte   %00000001
    .byte   %00000000
Star1:
    .byte   %11100001
    .byte   %00010001
    .byte   %00001001
    .byte   %00001001
    .byte   %00001001
    .byte   %00010001
    .byte   %00011111
    .byte   %00000000
    .byte   %00000000
    .byte   %11111111
Star2:
    .byte   %11100011
    .byte   %00100010
    .byte   %00100001
    .byte   %00100001
    .byte   %00100001
    .byte   %00100000
    .byte   %00111110
    .byte   %00000010
    .byte   %00000010
    .byte   %11111110
Star3:
    .byte   %11000111
    .byte   %01000100
    .byte   %01111101
    .byte   %00000001
    .byte   %00111001
    .byte   %10101010
    .byte   %10101010
    .byte   %10010010
    .byte   %01000100
    .byte   %01111100
Star4:
    .byte   %10111100
    .byte   %10100100
    .byte   %00100101
    .byte   %00100110
    .byte   %00100000
    .byte   %00100111
    .byte   %00100100
    .byte   %00100111
    .byte   %00100000
    .byte   %00111111
Star5:
    .byte   %01111111
    .byte   %10000001
    .byte   %00000001
    .byte   %00111111
    .byte   %00100000
    .byte   %10010000
    .byte   %01010000
    .byte   %10010000
    .byte   %00100000
    .byte   %11000000

Fire0:
    .byte   %00011110
    .byte   %00010010
    .byte   %00010010
    .byte   %00010010
    .byte   %00010011
    .byte   %00010011
    .byte   %00010010
    .byte   %00010011
    .byte   %00010000
    .byte   %00011111
Fire1:
    .byte   %00000111
    .byte   %00000100
    .byte   %00000100
    .byte   %00000111
    .byte   %10000001
    .byte   %10000001
    .byte   %00000001
    .byte   %11110111
    .byte   %00010100
    .byte   %11110111
Fire2:
    .byte   %11111011
    .byte   %00001010
    .byte   %00001010
    .byte   %00111010
    .byte   %00100010
    .byte   %00100010
    .byte   %00100010
    .byte   %00111010
    .byte   %00001010
    .byte   %11111011
Fire3:
    .byte   %11000111
    .byte   %01001000
    .byte   %01010000
    .byte   %01100011
    .byte   %00000010
    .byte   %01111001
    .byte   %01000101
    .byte   %01111001
    .byte   %00000010
    .byte   %11111100
Fire4:
    .byte   %11111111
    .byte   %00000000
    .byte   %00000000
    .byte   %11100111
    .byte   %00100100
    .byte   %00100111
    .byte   %00100100
    .byte   %00100111
    .byte   %00100000
    .byte   %00111111
Fire5:
    .byte   %11111000
    .byte   %00001000
    .byte   %00001000
    .byte   %11111000
    .byte   %00000000
    .byte   %00000000
    .byte   %00000000
    .byte   %11111000
    .byte   %00001000
    .byte   %11111000

Score0:
Score5:
Xype0:
Xype5:
    .byte   %00000000
    .byte   %00000000
    .byte   %00000000
    .byte   %00000000
    .byte   %00000000
    .byte   %00000000
    .byte   %00000000
Xype1:
    .byte   %00110011
    .byte   %00011011
    .byte   %00001110
    .byte   %00000110
    .byte   %00000111
    .byte   %00001101
    .byte   %00001100

Xype2:
    .byte   %00011001
    .byte   %00011001
    .byte   %00001100
    .byte   %00001100
    .byte   %00011110
    .byte   %10110011
    .byte   %11100011

Xype3:
    .byte   %10000011
    .byte   %10000011
    .byte   %11111001
    .byte   %11001101
    .byte   %11001101
    .byte   %01101100
    .byte   %01111000
Xype4:
    .byte   %11110000
    .byte   %00000000
    .byte   %10000000
    .byte   %11111000
    .byte   %10000000
    .byte   %11000000
    .byte   %11111100

presents0:
    .byte   %00001000
    .byte   %00001110
    .byte   %00001001
    .byte   %00001001
    .byte   %00001001
    .byte   %00001110
    .byte   %00000000
presents1:
    .byte   %00000000
    .byte   %01000011
    .byte   %01000100
    .byte   %01000111
    .byte   %01000100
    .byte   %00110011
    .byte   %00000000
presents2:
    .byte   %00000000
    .byte   %10011100
    .byte   %00000010
    .byte   %10001100
    .byte   %01010000
    .byte   %10001110
    .byte   %00000000
presents3:
    .byte   %00000000
    .byte   %01110010
    .byte   %10000010
    .byte   %11110010
    .byte   %10001010
    .byte   %01110011
    .byte   %00000000
presents4:
    .byte   %00000000
    .byte   %01000101
    .byte   %01001000
    .byte   %01001000
    .byte   %01001001
    .byte   %10011100
    .byte   %00001000
presents5:
    .byte   %00000000
    .byte   %11000000
    .byte   %00100000
    .byte   %11000000
    .byte   %00000000
    .byte   %11100000
    .byte   %00000000

Score1:
    .byte   %00111110
    .byte   %00000001
    .byte   %00000001
    .byte   %00011110
    .byte   %00100000
    .byte   %00100000
    .byte   %00011111

Score2:
    .byte   %00110011
    .byte   %01000100
    .byte   %01000100
    .byte   %01000100
    .byte   %00110011
    .byte   %00000000
    .byte   %00000000

Score3:
    .byte   %00100001
    .byte   %10100010
    .byte   %10100011
    .byte   %10100010
    .byte   %00011001 
    .byte   %00000000
    .byte   %00000000

Score4:
    .byte   %11000000
    .byte   %00000100
    .byte   %11000000
    .byte   %00100100
    .byte   %11000000
    .byte   %00000000
    .byte   %00000000

PatternTab:
    .word Star0, Star1, Star2, Star3, Star4, Star5
    .word Fire0, Fire1, Fire2, Fire3, Fire4, Fire5
    .word Xype0, Xype1, Xype2, Xype3, Xype4, Xype5
    .word presents0, presents1, presents2, presents3, presents4, presents5
    .word logo, logo+INTROLINES, logo+INTROLINES*2, logo+INTROLINES*3, logo+INTROLINES*4, logo+INTROLINES*5
    .word Score0, Score1, Score2, Score3, Score4, Score5
    .word Meteors0, Meteors1, Meteors2, Meteors3, Meteors4, Meteors5
    .word Wave0, Wave1, Wave2, Wave3

zero
       .byte $38 ; |  XXX   | $F004
       .byte $44 ; | X   X  | $F005
       .byte $44 ; | X   X  | $F006
       .byte $44 ; | X   X  | $F007
       .byte $44 ; | X   X  | $F008
       .byte $44 ; | X   X  | $F009
       .byte $38 ; |  XXX   | $F00A

one
       .byte $38 ; |  XXX   | $F00C
       .byte $10 ; |   X    | $F00D
       .byte $10 ; |   X    | $F00E
       .byte $10 ; |   X    | $F00F
       .byte $10 ; |   X    | $F010
       .byte $30 ; |  XX    | $F011
       .byte $10 ; |   X    | $F012

two
       .byte $7C ; | XXXXX  | $F014
       .byte $40 ; | X      | $F015
       .byte $40 ; | X      | $F016
       .byte $38 ; |  XXX   | $F017
       .byte $04 ; |     X  | $F018
       .byte $44 ; | X   X  | $F019
       .byte $38 ; |  XXX   | $F01A

three
       .byte $38 ; |  XXX   | $F01C
       .byte $44 ; | X   X  | $F01D
       .byte $04 ; |     X  | $F01E
       .byte $18 ; |   XX   | $F01F
       .byte $04 ; |     X  | $F020
       .byte $44 ; | X   X  | $F021
       .byte $38 ; |  XXX   | $F022

four
       .byte $08 ; |    X   | $F024
       .byte $08 ; |    X   | $F025
       .byte $7C ; | XXXXX  | $F026
       .byte $48 ; | X  X   | $F027
       .byte $28 ; |  X X   | $F028
       .byte $18 ; |   XX   | $F029
       .byte $08 ; |    X   | $F02A

five
       .byte $38 ; |  XXX   | $F02C
       .byte $44 ; | X   X  | $F02D
       .byte $04 ; |     X  | $F02E
       .byte $04 ; |     X  | $F02F
       .byte $78 ; | XXXX   | $F030
       .byte $40 ; | X      | $F031
       .byte $7C ; | XXXXX  | $F032

six
       .byte $38 ; |  XXX   | $F034
       .byte $44 ; | X   X  | $F035
       .byte $44 ; | X   X  | $F036
       .byte $78 ; | XXXX   | $F037
       .byte $40 ; | X      | $F038
       .byte $20 ; |  X     | $F039
       .byte $1C ; |   XXX  | $F03A

seven
       .byte $20 ; |  X     | $F03C
       .byte $20 ; |  X     | $F03D
       .byte $20 ; |  X     | $F03E
       .byte $10 ; |   X    | $F03F
       .byte $08 ; |    X   | $F040
       .byte $04 ; |     X  | $F041
       .byte $7C ; | XXXXX  | $F042

eight
       .byte $38 ; |  XXX   | $F044
       .byte $44 ; | X   X  | $F045
       .byte $44 ; | X   X  | $F046
       .byte $38 ; |  XXX   | $F047
       .byte $44 ; | X   X  | $F048
       .byte $44 ; | X   X  | $F049
       .byte $38 ; |  XXX   | $F04A

nine
       .byte $70 ; | XXX    | $F04C
       .byte $08 ; |    X   | $F04D
       .byte $04 ; |     X  | $F04E
       .byte $3C ; |  XXXX  | $F04F
       .byte $44 ; | X   X  | $F050
       .byte $44 ; | X   X  | $F051
       .byte $38 ; |  XXX   | $F052

Meteors0:
    .byte   %00001000
    .byte   %00001000
    .byte   %00001000
    .byte   %00001001
    .byte   %00001010
    .byte   %00001100
    .byte   %00001000

Meteors1:
    .byte   %00100111
    .byte   %00101000
    .byte   %00101111
    .byte   %00101000
    .byte   %10100111
    .byte   %01100000
    .byte   %00100000

Meteors2:
    .byte   %00001001
    .byte   %00010010
    .byte   %00010011
    .byte   %10010010
    .byte   %00111001
    .byte   %00010000
    .byte   %00000000

Meteors3:
    .byte   %11000110
    .byte   %00001001
    .byte   %11001001
    .byte   %00101001
    .byte   %11000110
    .byte   %00000000
    .byte   %00000000

Meteors4:
    .byte   %01000111
    .byte   %01000000
    .byte   %01000011
    .byte   %01000100
    .byte   %00110011
    .byte   %00000000
    .byte   %00000000

Meteors5:
    .byte   %00010000
    .byte   %10000000
    .byte   %00010000
    .byte   %00010000
    .byte   %10010000
    .byte   %00010000
    .byte   %00010000

Wave0:
    .byte   %10000010
    .byte   %11000110
    .byte   %10101010
    .byte   %10010010
    .byte   %10000010
    .byte   %10000010
    .byte   %10000010

Wave1:
    .byte   %01110100
    .byte   %10001001
    .byte   %01111001
    .byte   %00001010
    .byte   %11110010
    .byte   %00000000
    .byte   %00000000


Wave2:
    .byte   %10000111
    .byte   %01001000
    .byte   %01001111
    .byte   %00101000
    .byte   %00100111
    .byte   %00000000
    .byte   %00000000

Wave3:
    .byte   %00000000
    .byte   %00010000
    .byte   %00000000
    .byte   %10010000
    .byte   %00000000
    .byte   %00000000
    .byte   %00000000

digittab
    .byte #<zero,#<one,#<two,#<three,#<four
    .byte #<five,#<six,#<seven,#<eight,#<nine

BitTab
    .byte   0, 6, 2, 2, 5
    .byte   3, 5, 6, 1
RowTab
    .byte   3, 0, 4, 0, 5
    .byte   0, 0, 0, 5
ColTab
    .byte   0, 2, 3, 4, 4
    .byte   0, 1, 2, 3

DirXTab:
    .byte    0, 1, 1, 1, 0,-1,-1,-1
DirYTab:
    .byte    1, 1, 0,-1,-1,-1, 0, 1

MultTab:
    .byte   0, INTROLINES*1, INTROLINES*2, INTROLINES*3, INTROLINES*4, INTROLINES*5
ScanTab:
    .byte   -2, 2,-1, 2, 1, 1, 1    ; even-direction (-2, 0,-1, 1, 2, 3)
ScanTabOdd:
    .byte   -1,-1, 2, 1, 1, 1, 1    ; odd -direction (-1,-2, 0, 1, 2, 3)

    ORG     $2FF0
    RORG    $FFF0

SwitchToBank0:
    BIT $1FF8
    JMP StartScore
    .byte   $FF, $FF, $FF, $FF  ;           addresses for bank-switching (do not use!)
    .word   start
    .word   start
    .word   start